@@ -10,8 +10,8 @@ module Config
   DESTDIR = '' unless defined? DESTDIR
   CONFIG = {}
   CONFIG["DESTDIR"] = DESTDIR
-  CONFIG["PATCHLEVEL"] = "370"
-  CONFIG["INSTALL"] = '/usr/bin/install -c -o root -g bin'
+  CONFIG["PATCHLEVEL"] = "374"
+  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-1.8.7-p374/bin/install -c '
   CONFIG["EXEEXT"] = ""
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
   CONFIG["ruby_install_name"] = "ruby18"
@@ -19,7 +19,7 @@ module Config
   CONFIG["RUBY_SO_NAME"] = "ruby18"
   CONFIG["MANTYPE"] = "man"
   CONFIG["NROFF"] = "/bin/false"
-  CONFIG["configure_args"] = " '--program-suffix=18' '--enable-ipv6' '--with-dbm-type=bogus' '--with-opt-dir=/usr/local' '--disable-option-checking' '--with-tcl-include=/usr/local/include/tcl8.5' '--with-tk-include=/usr/local/include/tk8.5' '--with-X11-dir=/usr/X11R6' '--enable-shared' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' 'CC=cc' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'YACC=yacc'"
+  CONFIG["configure_args"] = " '--program-suffix=18' '--enable-ipv6' '--with-dbm-type=bogus' '--with-opt-dir=/usr/local' '--disable-option-checking' '--enable-shared' '--with-tcl-include=/usr/local/include/tcl8.5' '--with-tk-include=/usr/local/include/tk8.5' '--with-X11-dir=/usr/X11R6' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'YACC=yacc'"
   CONFIG["vendordir"] = "$(libdir)/ruby/vendor_ruby"
   CONFIG["sitedir"] = "$(libdir)/ruby/site_ruby"
   CONFIG["sitearch"] = "alpha-openbsd"
@@ -72,9 +72,9 @@ module Config
   CONFIG["MAKEDIRS"] = "mkdir -p"
   CONFIG["CP"] = "cp"
   CONFIG["RM"] = "rm -f"
-  CONFIG["INSTALL_DATA"] = "install -c -o root -g bin -m 444"
-  CONFIG["INSTALL_SCRIPT"] = "install -c -o root -g bin -m 555"
-  CONFIG["INSTALL_PROGRAM"] = "install -c -s -o root -g bin -m 555"
+  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-1.8.7-p374/bin/install -c  -m 644"
+  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-1.8.7-p374/bin/install -c  -m 755"
+  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-1.8.7-p374/bin/install -c -s  -m 755"
   CONFIG["SET_MAKE"] = ""
   CONFIG["LN_S"] = "ln -s"
   CONFIG["OBJDUMP"] = ""
@@ -98,18 +98,18 @@ module Config
   CONFIG["LDFLAGS"] = "-L. -L/usr/local/lib -Wl,-E"
   CONFIG["CFLAGS"] = "-mieee -O2 -pipe  -fPIC $(cflags)"
   CONFIG["CC"] = "cc"
-  CONFIG["target_os"] = "openbsd5.3"
+  CONFIG["target_os"] = "openbsd6.0"
   CONFIG["target_vendor"] = "unknown"
   CONFIG["target_cpu"] = "alpha"
-  CONFIG["target"] = "alpha-unknown-openbsd5.3"
-  CONFIG["host_os"] = "openbsd5.3"
+  CONFIG["target"] = "alpha-unknown-openbsd6.0"
+  CONFIG["host_os"] = "openbsd6.0"
   CONFIG["host_vendor"] = "unknown"
   CONFIG["host_cpu"] = "alpha"
-  CONFIG["host"] = "alpha-unknown-openbsd5.3"
-  CONFIG["build_os"] = "openbsd5.3"
+  CONFIG["host"] = "alpha-unknown-openbsd6.0"
+  CONFIG["build_os"] = "openbsd6.0"
   CONFIG["build_vendor"] = "unknown"
   CONFIG["build_cpu"] = "alpha"
-  CONFIG["build"] = "alpha-unknown-openbsd5.3"
+  CONFIG["build"] = "alpha-unknown-openbsd6.0"
   CONFIG["TEENY"] = "7"
   CONFIG["MINOR"] = "8"
   CONFIG["MAJOR"] = "1"
@@ -182,3 +182,23 @@ module Config
 end
 RbConfig = Config # compatibility for ruby-1.9
 CROSS_COMPILING = nil unless defined? CROSS_COMPILING
+install = '/usr/bin/install -c -o root -g bin'  
+c = RbConfig::CONFIG 
+m = RbConfig::MAKEFILE_CONFIG
+if v = ENV['BSD_INSTALL_SCRIPT']
+  m['INSTALL'] = c['INSTALL'] = v.sub('-m 755', '')
+  m['INSTALL_SCRIPT'] = c['INSTALL_SCRIPT'] = v
+else
+  m['INSTALL'] = c['INSTALL'] = "#{install}"
+  m['INSTALL_SCRIPT'] = c['INSTALL_SCRIPT'] = "#{install} -m 755"
+end
+if v = ENV['BSD_INSTALL_PROGRAM']
+  m['INSTALL_PROGRAM'] = c['INSTALL_PROGRAM'] = v
+else
+  m['INSTALL_PROGRAM'] = c['INSTALL_PROGRAM'] = "#{install} -s -m 755"
+end
+if v = ENV['BSD_INSTALL_DATA']
+  m['INSTALL_DATA'] = c['INSTALL_DATA'] = v
+else
+  m['INSTALL_DATA'] = c['INSTALL_DATA'] = "#{install} -m 644"
+end
