@@ -1,173 +1,33 @@
 # = net/pop.rb
 #
-#--
-# Copyright (c) 1999-2003 Yukihiro Matsumoto
-# Copyright (c) 1999-2003 Minero Aoki
+# Copyright (c) 1999-2007 Yukihiro Matsumoto.
+#
+# Copyright (c) 1999-2007 Minero Aoki.
 # 
-# written & maintained by Minero Aoki <aamine@loveruby.net>
+# Written & maintained by Minero Aoki <aamine@loveruby.net>.
+#
+# Documented by William Webber and Minero Aoki.
 # 
 # This program is free software. You can re-distribute and/or
 # modify this program under the same terms as Ruby itself,
-# Ruby Distribute License or GNU General Public License.
+# Ruby Distribute License.
 # 
-# NOTE: You can find Japanese version of this document in
-# the doc/net directory of the standard ruby interpreter package.
+# NOTE: You can find Japanese version of this document at:
+# http://www.ruby-lang.org/ja/man/html/net_pop.html
 # 
-# $Id: pop.rb,v 1.62 2003/09/29 18:12:20 aamine Exp $
-#++
+#   $Id: pop.rb 29903 2010-11-24 07:38:32Z shyouhei $
+# 
+# See Net::POP3 for documentation.
 #
-# == What is This Library?
-# 
-# This library provides functionality for retrieving 
-# email via POP3, the Post Office Protocol version 3. For details
-# of POP3, see [RFC1939] (http://www.ietf.org/rfc/rfc1939.txt).
-# 
-# == Examples
-# 
-# === Retrieving Messages 
-# 
-# This example retrieves messages from the server and deletes them 
-# on the server.
-# Messages are written to files named 'inbox/1', 'inbox/2', ....
-# Replace 'pop.example.com' with your POP3 server address, and
-# 'YourAccount' and 'YourPassword' with the appropriate account
-# details.
-# 
-#     require 'net/pop'
-# 
-#     pop = Net::POP3.new('pop.example.com')
-#     pop.start('YourAccount', 'YourPassword')             # (1)
-#     if pop.mails.empty?
-#       puts 'no mail.'
-#     else
-#       i = 0
-#       pop.each_mail do |m|   # or "pop.mails.each ..."   # (2)
-#         File.open("inbox/#{i}", 'w') {|f|
-#           f.write m.pop
-#         }
-#         m.delete
-#         i += 1
-#       end
-#       puts "#{pop.mails.size} mails popped."
-#     end
-#     pop.finish                                           # (3)
-# 
-# 1. call Net::POP3#start and start POP session
-# 2. access messages by using POP3#each_mail and/or POP3#mails
-# 3. close POP session by calling POP3#finish or use the block form of #start.
-# 
-# === Shortened Code
-# 
-# The example above is very verbose. You can shorten the code by using
-# some utility methods. First, the block form of Net::POP3.start can
-# be used instead of POP3.new, POP3#start and POP3#finish.
-# 
-#     require 'net/pop'
-# 
-#     Net::POP3.start('pop.example.com', 110,
-#                     'YourAccount', 'YourPassword') {|pop|
-#       if pop.mails.empty?
-#         puts 'no mail.'
-#       else
-#         i = 0
-#         pop.each_mail do |m|   # or "pop.mails.each ..."
-#           File.open("inbox/#{i}", 'w') {|f|
-#             f.write m.pop
-#           }
-#           m.delete
-#           i += 1
-#         end
-#         puts "#{pop.mails.size} mails popped."
-#       end
-#     }
-# 
-# POP3#delete_all alternates #each_mail and m.delete.
-# 
-#     require 'net/pop'
-# 
-#     Net::POP3.start('pop.example.com', 110,
-#                     'YourAccount', 'YourPassword') {|pop|
-#       if pop.mails.empty?
-#         puts 'no mail.'
-#       else
-#         i = 1
-#         pop.delete_all do |m|
-#           File.open("inbox/#{i}", 'w') {|f|
-#             f.write m.pop
-#           }
-#           i += 1
-#         end
-#       end
-#     }
-# 
-# And here is an even shorter example.
-# 
-#     require 'net/pop'
-# 
-#     i = 0
-#     Net::POP3.delete_all('pop.example.com', 110,
-#                          'YourAccount', 'YourPassword') do |m|
-#       File.open("inbox/#{i}", 'w') {|f|
-#         f.write m.pop
-#       }
-#       i += 1
-#     end
-# 
-# === Memory Space Issues
-# 
-# All the examples above get each message as one big string.
-# This example avoids this.
-# 
-#     require 'net/pop'
-# 
-#     i = 1
-#     Net::POP3.delete_all('pop.example.com', 110,
-#                          'YourAccount', 'YourPassword') do |m|
-#       File.open("inbox/#{i}", 'w') {|f|
-#         m.pop do |chunk|    # get a message little by little.
-#           f.write chunk
-#         end
-#         i += 1
-#       }
-#     end
-# 
-# === Using APOP
-# 
-# The net/pop library supports APOP authentication.
-# To use APOP, use the Net::APOP class instead of the Net::POP3 class.
-# You can use the utility method, Net::POP3.APOP(). For example:
-# 
-#     require 'net/pop'
-# 
-#     # Use APOP authentication if $isapop == true
-#     pop = Net::POP3.APOP($is_apop).new('apop.example.com', 110)
-#     pop.start(YourAccount', 'YourPassword') {|pop|
-#       # Rest code is same.
-#     }
-# 
-# === Fetch Only Selected Mail Using `UIDL' POP Command
-# 
-# If your POP server provides UIDL functionality,
-# you can grab only selected mails from the POP server.
-# e.g.
-# 
-#     def need_pop?( id )
-#       # determine if we need pop this mail...
-#     end
-# 
-#     Net::POP3.start('pop.example.com', 110,
-#                     'Your account', 'Your password') {|pop|
-#       pop.mails.select {|m| need_pop?(m.unique_id) }.each do |m|
-#         do_something(m.pop)
-#       end
-#     }
-# 
-# The POPMail#unique_id() method returns the unique-id of the message as a
-# String. Normally the unique-id is a hash of the message.
-# 
 
 require 'net/protocol'
 require 'digest/md5'
+require 'timeout'
+
+begin
+  require "openssl/ssl"
+rescue LoadError
+end
 
 module Net
 
@@ -181,24 +41,180 @@ module Net
   # Unexpected response from the server.
   class POPBadResponse < POPError; end
 
-
   #
-  # Class providing POP3 client functionality.
+  # = Net::POP3
   #
-  # See documentation for the file pop.rb for examples of usage.
+  # == What is This Library?
+  # 
+  # This library provides functionality for retrieving 
+  # email via POP3, the Post Office Protocol version 3. For details
+  # of POP3, see [RFC1939] (http://www.ietf.org/rfc/rfc1939.txt).
+  # 
+  # == Examples
+  # 
+  # === Retrieving Messages 
+  # 
+  # This example retrieves messages from the server and deletes them 
+  # on the server.
   #
+  # Messages are written to files named 'inbox/1', 'inbox/2', ....
+  # Replace 'pop.example.com' with your POP3 server address, and
+  # 'YourAccount' and 'YourPassword' with the appropriate account
+  # details.
+  # 
+  #     require 'net/pop'
+  # 
+  #     pop = Net::POP3.new('pop.example.com')
+  #     pop.start('YourAccount', 'YourPassword')             # (1)
+  #     if pop.mails.empty?
+  #       puts 'No mail.'
+  #     else
+  #       i = 0
+  #       pop.each_mail do |m|   # or "pop.mails.each ..."   # (2)
+  #         File.open("inbox/#{i}", 'w') do |f|
+  #           f.write m.pop
+  #         end
+  #         m.delete
+  #         i += 1
+  #       end
+  #       puts "#{pop.mails.size} mails popped."
+  #     end
+  #     pop.finish                                           # (3)
+  # 
+  # 1. Call Net::POP3#start and start POP session.
+  # 2. Access messages by using POP3#each_mail and/or POP3#mails.
+  # 3. Close POP session by calling POP3#finish or use the block form of #start.
+  # 
+  # === Shortened Code
+  # 
+  # The example above is very verbose. You can shorten the code by using
+  # some utility methods. First, the block form of Net::POP3.start can
+  # be used instead of POP3.new, POP3#start and POP3#finish.
+  # 
+  #     require 'net/pop'
+  # 
+  #     Net::POP3.start('pop.example.com', 110,
+  #                     'YourAccount', 'YourPassword') do |pop|
+  #       if pop.mails.empty?
+  #         puts 'No mail.'
+  #       else
+  #         i = 0
+  #         pop.each_mail do |m|   # or "pop.mails.each ..."
+  #           File.open("inbox/#{i}", 'w') do |f|
+  #             f.write m.pop
+  #           end
+  #           m.delete
+  #           i += 1
+  #         end
+  #         puts "#{pop.mails.size} mails popped."
+  #       end
+  #     end
+  # 
+  # POP3#delete_all is an alternative for #each_mail and #delete.
+  # 
+  #     require 'net/pop'
+  # 
+  #     Net::POP3.start('pop.example.com', 110,
+  #                     'YourAccount', 'YourPassword') do |pop|
+  #       if pop.mails.empty?
+  #         puts 'No mail.'
+  #       else
+  #         i = 1
+  #         pop.delete_all do |m|
+  #           File.open("inbox/#{i}", 'w') do |f|
+  #             f.write m.pop
+  #           end
+  #           i += 1
+  #         end
+  #       end
+  #     end
+  # 
+  # And here is an even shorter example.
+  # 
+  #     require 'net/pop'
+  # 
+  #     i = 0
+  #     Net::POP3.delete_all('pop.example.com', 110,
+  #                          'YourAccount', 'YourPassword') do |m|
+  #       File.open("inbox/#{i}", 'w') do |f|
+  #         f.write m.pop
+  #       end
+  #       i += 1
+  #     end
+  # 
+  # === Memory Space Issues
+  # 
+  # All the examples above get each message as one big string.
+  # This example avoids this.
+  # 
+  #     require 'net/pop'
+  # 
+  #     i = 1
+  #     Net::POP3.delete_all('pop.example.com', 110,
+  #                          'YourAccount', 'YourPassword') do |m|
+  #       File.open("inbox/#{i}", 'w') do |f|
+  #         m.pop do |chunk|    # get a message little by little.
+  #           f.write chunk
+  #         end
+  #         i += 1
+  #       end
+  #     end
+  # 
+  # === Using APOP
+  # 
+  # The net/pop library supports APOP authentication.
+  # To use APOP, use the Net::APOP class instead of the Net::POP3 class.
+  # You can use the utility method, Net::POP3.APOP(). For example:
+  # 
+  #     require 'net/pop'
+  # 
+  #     # Use APOP authentication if $isapop == true
+  #     pop = Net::POP3.APOP($is_apop).new('apop.example.com', 110)
+  #     pop.start(YourAccount', 'YourPassword') do |pop|
+  #       # Rest of the code is the same.
+  #     end
+  # 
+  # === Fetch Only Selected Mail Using 'UIDL' POP Command
+  # 
+  # If your POP server provides UIDL functionality,
+  # you can grab only selected mails from the POP server.
+  # e.g.
+  # 
+  #     def need_pop?( id )
+  #       # determine if we need pop this mail...
+  #     end
+  # 
+  #     Net::POP3.start('pop.example.com', 110,
+  #                     'Your account', 'Your password') do |pop|
+  #       pop.mails.select { |m| need_pop?(m.unique_id) }.each do |m|
+  #         do_something(m.pop)
+  #       end
+  #     end
+  # 
+  # The POPMail#unique_id() method returns the unique-id of the message as a
+  # String. Normally the unique-id is a hash of the message.
+  # 
   class POP3 < Protocol
 
-    Revision = %q$Revision: 1.62 $.split[1]
+    Revision = %q$Revision: 29903 $.split[1]
 
     #
     # Class Parameters
     #
 
-    # The default port for POP3 connections, port 110
     def POP3.default_port
+      default_pop3_port()
+    end
+
+    # The default port for POP3 connections, port 110
+    def POP3.default_pop3_port
       110
     end
+    
+    # The default port for POP3S connections, port 995
+    def POP3.default_pop3s_port
+      995
+    end
 
     def POP3.socket_type   #:nodoc: obsolete
       Net::InternetMessageIO
@@ -215,11 +231,11 @@ module Net
     #     pop = Net::POP3::APOP($is_apop).new(addr, port)
     #
     #     # Example 2
-    #     Net::POP3::APOP($is_apop).start(addr, port) {|pop|
+    #     Net::POP3::APOP($is_apop).start(addr, port) do |pop|
     #       ....
-    #     }
+    #     end
     #
-    def POP3.APOP( isapop )
+    def POP3.APOP(isapop)
       isapop ? APOP : POP3
     end
 
@@ -227,24 +243,25 @@ module Net
     # yielding it to the +block+.
     # This method is equivalent to:
     #
-    #     Net::POP3.start(address, port, account, password) {|pop|
+    #     Net::POP3.start(address, port, account, password) do |pop|
     #       pop.each_mail do |m|
     #         yield m
     #       end
-    #     }
+    #     end
     #
     # This method raises a POPAuthenticationError if authentication fails.
     #
-    #     # Typical usage
+    # === Example
+    #
     #     Net::POP3.foreach('pop.example.com', 110,
     #                       'YourAccount', 'YourPassword') do |m|
     #       file.write m.pop
     #       m.delete if $DELETE
     #     end
     #
-    def POP3.foreach( address, port = nil,
-                      account = nil, password = nil,
-                      isapop = false, &block )  # :yields: message
+    def POP3.foreach(address, port = nil,
+                     account = nil, password = nil,
+                     isapop = false, &block)  # :yields: message
       start(address, port, account, password, isapop) {|pop|
         pop.each_mail(&block)
       }
@@ -256,15 +273,16 @@ module Net
     #
     # This method raises a POPAuthenticationError if authentication fails.
     #
-    #     # Example
+    # === Example
+    #
     #     Net::POP3.delete_all('pop.example.com', 110,
     #                          'YourAccount', 'YourPassword') do |m|
     #       file.write m.pop
     #     end
     #
-    def POP3.delete_all( address, port = nil,
-                         account = nil, password = nil,
-                         isapop = false, &block )
+    def POP3.delete_all(address, port = nil,
+                        account = nil, password = nil,
+                        isapop = false, &block)
       start(address, port, account, password, isapop) {|pop|
         pop.delete_all(&block)
       }
@@ -274,66 +292,127 @@ module Net
     #
     # This method raises POPAuthenticationError if authentication fails.
     #
-    #     # Example 1: normal POP3
+    # === Example: normal POP3
+    #
     #     Net::POP3.auth_only('pop.example.com', 110,
     #                         'YourAccount', 'YourPassword')
     #
-    #     # Example 2: APOP
+    # === Example: APOP
+    #
     #     Net::POP3.auth_only('pop.example.com', 110,
     #                         'YourAccount', 'YourPassword', true)
     #
-    def POP3.auth_only( address, port = nil,
-                        account = nil, password = nil,
-                        isapop = false )
+    def POP3.auth_only(address, port = nil,
+                       account = nil, password = nil,
+                       isapop = false)
       new(address, port, isapop).auth_only account, password
     end
 
     # Starts a pop3 session, attempts authentication, and quits.
     # This method must not be called while POP3 session is opened.
     # This method raises POPAuthenticationError if authentication fails.
-    def auth_only( account, password )
-      raise IOError, 'opening already opened POP session' if started?
+    def auth_only(account, password)
+      raise IOError, 'opening previously opened POP session' if started?
       start(account, password) {
         ;
       }
     end
 
+    #
+    # SSL
+    #
+
+    @ssl_params = nil
+
+    # call-seq:
+    #    Net::POP.enable_ssl(params = {})
+    #
+    # Enable SSL for all new instances.
+    # +params+ is passed to OpenSSL::SSLContext#set_params.
+    def POP3.enable_ssl(*args)
+      @ssl_params = create_ssl_params(*args)
+    end
+
+    def POP3.create_ssl_params(verify_or_params = {}, certs = nil)
+      begin
+        params = verify_or_params.to_hash
+      rescue NoMethodError
+        params = {}
+        params[:verify_mode] = verify_or_params
+        if certs
+          if File.file?(certs)
+            params[:ca_file] = certs
+          elsif File.directory?(certs)
+            params[:ca_path] = certs
+          end
+        end
+      end
+      return params
+    end
+
+    # Disable SSL for all new instances.
+    def POP3.disable_ssl
+      @ssl_params = nil
+    end
+
+    def POP3.ssl_params
+      return @ssl_params
+    end
+
+    def POP3.use_ssl?
+      return !@ssl_params.nil?
+    end
+
+    def POP3.verify
+      return @ssl_params[:verify_mode]
+    end
+
+    def POP3.certs
+      return @ssl_params[:ca_file] || @ssl_params[:ca_path]
+    end
+
     #
     # Session management
     #
 
     # Creates a new POP3 object and open the connection.  Equivalent to 
-    # Net::POP3.new(address, port, isapop).start(account, password)
+    #
+    #   Net::POP3.new(address, port, isapop).start(account, password)
     #
     # If +block+ is provided, yields the newly-opened POP3 object to it,
     # and automatically closes it at the end of the session.
     #
-    # Typical usage:
+    # === Example
     #
-    #    Net::POP3.start(addr, port, account, password) {|pop|
+    #    Net::POP3.start(addr, port, account, password) do |pop|
     #      pop.each_mail do |m|
     #        file.write m.pop
     #        m.delete
     #      end
-    #    }
+    #    end
     #
-    def POP3.start( address, port = nil,
-                    account = nil, password = nil,
-                    isapop = false, &block ) # :yield: pop
+    def POP3.start(address, port = nil,
+                   account = nil, password = nil,
+                   isapop = false, &block)   # :yield: pop
       new(address, port, isapop).start(account, password, &block)
     end
-
+    
     # Creates a new POP3 object.
+    #
     # +address+ is the hostname or ip address of your POP3 server.
-    # The optional +port+ is the port to connect to; it defaults to 110.
+    #
+    # The optional +port+ is the port to connect to.
+    #
     # The optional +isapop+ specifies whether this connection is going
     # to use APOP authentication; it defaults to +false+.
+    #
     # This method does *not* open the TCP connection.
-    def initialize( addr, port = nil, isapop = false )
+    def initialize(addr, port = nil, isapop = false)
       @address = addr
-      @port = port || self.class.default_port
+      @ssl_params = POP3.ssl_params
+      @port = port
       @apop = isapop
-
+      
       @command = nil
       @socket = nil
       @started = false
@@ -351,6 +430,32 @@ module Net
       @apop
     end
 
+    # does this instance use SSL?
+    def use_ssl?
+      return !@ssl_params.nil?
+    end
+   
+    # call-seq:
+    #    Net::POP#enable_ssl(params = {})
+    #
+    # Enables SSL for this instance.  Must be called before the connection is
+    # established to have any effect.
+    # +params[:port]+ is port to establish the SSL connection on; Defaults to 995.
+    # +params+ (except :port) is passed to OpenSSL::SSLContext#set_params.
+    def enable_ssl(verify_or_params = {}, certs = nil, port = nil)
+      begin
+        @ssl_params = verify_or_params.to_hash.dup
+        @port = @ssl_params.delete(:port) || @port
+      rescue NoMethodError
+        @ssl_params = POP3.create_ssl_params(verify_or_params, certs)
+        @port = port || @port
+      end
+    end
+    
+    def disable_ssl
+      @ssl_params = nil
+    end
+
     # Provide human-readable stringification of class state.
     def inspect
       "#<#{self.class} #{@address}:#{@port} open=#{@started}>"
@@ -361,14 +466,15 @@ module Net
     #
     # Set an output stream for debugging.
     #
-    #   # Example
+    # === Example
+    #
     #   pop = Net::POP.new(addr, port)
     #   pop.set_debug_output $stderr
-    #   pop.start(account, passwd) {
+    #   pop.start(account, passwd) do |pop|
     #     ....
-    #   }
+    #   end
     #
-    def set_debug_output( arg )
+    def set_debug_output(arg)
       @debug_output = arg
     end
 
@@ -376,7 +482,9 @@ module Net
     attr_reader :address
 
     # The port number to connect to.
-    attr_reader :port
+    def port
+      return @port || (use_ssl? ? POP3.default_pop3s_port : POP3.default_pop3_port)
+    end
 
     # Seconds to wait until a connection is opened.
     # If the POP3 object cannot open a connection within this time,
@@ -389,7 +497,7 @@ module Net
     attr_reader :read_timeout
 
     # Set the read timeout.
-    def read_timeout=( sec )
+    def read_timeout=(sec)
       @command.socket.read_timeout = sec if @command
       @read_timeout = sec
     end
@@ -407,9 +515,8 @@ module Net
     # closes the session after block call finishes.
     #
     # This method raises a POPAuthenticationError if authentication fails.
-    def start( account, password ) # :yield: pop
+    def start(account, password) # :yield: pop
       raise IOError, 'POP session already started' if @started
-
       if block_given?
         begin
           do_start account, password
@@ -423,9 +530,23 @@ module Net
       end
     end
 
-    def do_start( account, password )
-      @socket = self.class.socket_type.open(@address, @port,
-                                   @open_timeout, @read_timeout, @debug_output)
+    def do_start(account, password)
+      s = timeout(@open_timeout) { TCPSocket.open(@address, port) }
+      if use_ssl?
+        raise 'openssl library not installed' unless defined?(OpenSSL)
+        context = OpenSSL::SSL::SSLContext.new
+        context.set_params(@ssl_params)
+        s = OpenSSL::SSL::SSLSocket.new(s, context)
+        s.sync_close = true
+        s.connect
+        if context.verify_mode != OpenSSL::SSL::VERIFY_NONE
+          s.post_connection_check(@address)
+        end
+      end
+      @socket = InternetMessageIO.new(s)
+      logging "POP session started: #{@address}:#{@port} (#{@apop ? 'APOP' : 'POP'})"
+      @socket.read_timeout = @read_timeout
+      @socket.debug_output = @debug_output
       on_connect
       @command = POP3Command.new(@socket)
       if apop?
@@ -435,7 +556,12 @@ module Net
       end
       @started = true
     ensure
-      do_finish if not @started
+      # Authentication failed, clean up connection.
+      unless @started
+        s.close if s and not s.closed?
+        @socket = nil
+        @command = nil
+      end
     end
     private :do_start
 
@@ -445,12 +571,14 @@ module Net
 
     # Finishes a POP3 session and closes TCP connection.
     def finish
-      raise IOError, 'POP session not started yet' unless started?
+      raise IOError, 'POP session not yet started' unless started?
       do_finish
     end
 
     def do_finish
       @mails = nil
+      @n_mails = nil
+      @n_bytes = nil
       @command.quit if @command
     ensure
       @started = false
@@ -513,7 +641,7 @@ module Net
     #   end
     #
     # This method raises a POPError if an error occurs.
-    def each_mail( &block )  # :yield: message
+    def each_mail(&block)  # :yield: message
       mails().each(&block)
     end
 
@@ -523,16 +651,18 @@ module Net
     #
     # If called with a block, yields each message in turn before deleting it.
     #
-    #     # Example
+    # === Example
+    #
     #     n = 1
     #     pop.delete_all do |m|
-    #       File.open("inbox/#{n}") {|f|
+    #       File.open("inbox/#{n}") do |f|
     #         f.write m.pop
-    #       }
+    #       end
     #       n += 1
     #     end
     #
     # This method raises a POPError if an error occurs.
+    #
     def delete_all # :yield: message
       mails().each do |m|
         yield m if block_given?
@@ -553,9 +683,12 @@ module Net
     end
 
     def set_all_uids   #:nodoc: internal use only (called from POPMail#uidl)
-      command().uidl.each do |num, uid|
-        @mails.find {|m| m.number == num }.uid = uid
-      end
+      uidl = command().uidl
+      @mails.each {|m| m.uid = uidl[m.number] }
+    end
+
+    def logging(msg)
+      @debug_output << msg + "\n" if @debug_output
     end
 
   end   # class POP3
@@ -585,7 +718,7 @@ module Net
   #
   class POPMail
 
-    def initialize( num, len, pop, cmd )   #:nodoc:
+    def initialize(num, len, pop, cmd)   #:nodoc:
       @number = num
       @length = len
       @pop = pop
@@ -606,40 +739,44 @@ module Net
       "#<#{self.class} #{@number}#{@deleted ? ' deleted' : ''}>"
     end
 
+    #
     # This method fetches the message.  If called with a block, the
     # message is yielded to the block one chunk at a time.  If called
     # without a block, the message is returned as a String.  The optional 
     # +dest+ argument will be prepended to the returned String; this
     # argument is essentially obsolete.
     #
-    #     # Example without block
+    # === Example without block
+    #
     #     POP3.start('pop.example.com', 110,
-    #                'YourAccount, 'YourPassword') {|pop|
+    #                'YourAccount, 'YourPassword') do |pop|
     #       n = 1
     #       pop.mails.each do |popmail|
-    #         File.open("inbox/#{n}", 'w') {|f|
+    #         File.open("inbox/#{n}", 'w') do |f|
     #           f.write popmail.pop              
-    #         }
+    #         end
     #         popmail.delete
     #         n += 1
     #       end
-    #     }
+    #     end
+    #
+    # === Example with block
     #
-    #     # Example with block
     #     POP3.start('pop.example.com', 110,
-    #                'YourAccount, 'YourPassword') {|pop|
+    #                'YourAccount, 'YourPassword') do |pop|
     #       n = 1
     #       pop.mails.each do |popmail|
-    #         File.open("inbox/#{n}", 'w') {|f|
+    #         File.open("inbox/#{n}", 'w') do |f|
     #           popmail.pop do |chunk|            ####
     #             f.write chunk
     #           end
-    #         }
+    #         end
     #         n += 1
     #       end
-    #     }
+    #     end
     #
     # This method raises a POPError if an error occurs.
+    #
     def pop( dest = '', &block ) # :yield: message_chunk
       if block_given?
         @command.retr(@number, &block)
@@ -656,10 +793,11 @@ module Net
     alias mail pop   #:nodoc: obsolete
 
     # Fetches the message header and +lines+ lines of body. 
+    #
     # The optional +dest+ argument is obsolete.
     #
     # This method raises a POPError if an error occurs.
-    def top( lines, dest = '' )
+    def top(lines, dest = '')
       @command.top(@number, lines) do |chunk|
         dest << chunk
       end
@@ -667,10 +805,11 @@ module Net
     end
 
     # Fetches the message header.     
+    #
     # The optional +dest+ argument is obsolete.
     #
     # This method raises a POPError if an error occurs.
-    def header( dest = '' )
+    def header(dest = '')
       top(0, dest)
     end
 
@@ -680,18 +819,19 @@ module Net
     #
     # This method raises a POPError if an error occurs.
     #
-    #     # Example
+    # === Example
+    #
     #     POP3.start('pop.example.com', 110,
-    #                'YourAccount, 'YourPassword') {|pop|
+    #                'YourAccount, 'YourPassword') do |pop|
     #       n = 1
     #       pop.mails.each do |popmail|
-    #         File.open("inbox/#{n}", 'w') {|f|
+    #         File.open("inbox/#{n}", 'w') do |f|
     #           f.write popmail.pop
-    #         }
+    #         end
     #         popmail.delete         ####
     #         n += 1
     #       end
-    #     }
+    #     end
     #
     def delete
       @command.dele @number
@@ -717,7 +857,7 @@ module Net
 
     alias uidl unique_id
 
-    def uid=( uid )   #:nodoc: internal use only (used from POP3#set_all_uids)
+    def uid=(uid)   #:nodoc: internal use only
       @uid = uid
     end
 
@@ -726,25 +866,25 @@ module Net
 
   class POP3Command   #:nodoc: internal use only
 
-    def initialize( sock )
+    def initialize(sock)
       @socket = sock
       @error_occured = false
       res = check_response(critical { recv_response() })
-      @apop_stamp = res.slice(/<.+>/)
+      @apop_stamp = res.slice(/<[!-~]+@[!-~]+>/)
     end
 
     def inspect
       "#<#{self.class} socket=#{@socket}>"
     end
 
-    def auth( account, password )
+    def auth(account, password)
       check_response_auth(critical {
-        check_response_auth(get_response('USER ' + account))
-        get_response('PASS ' + password)
+        check_response_auth(get_response('USER %s', account))
+        get_response('PASS %s', password)
       })
     end
 
-    def apop( account, password )
+    def apop(account, password)
       raise POPAuthenticationError, 'not APOP server; cannot login' \
                                                       unless @apop_stamp
       check_response_auth(critical {
@@ -775,28 +915,28 @@ module Net
     end
 
     def rset
-      check_response(critical { get_response 'RSET' })
+      check_response(critical { get_response('RSET') })
     end
 
-    def top( num, lines = 0, &block )
+    def top(num, lines = 0, &block)
       critical {
         getok('TOP %d %d', num, lines)
         @socket.each_message_chunk(&block)
       }
     end
 
-    def retr( num, &block )
+    def retr(num, &block)
       critical {
         getok('RETR %d', num)
         @socket.each_message_chunk(&block)
       }
     end
     
-    def dele( num )
+    def dele(num)
       check_response(critical { get_response('DELE %d', num) })
     end
 
-    def uidl( num = nil )
+    def uidl(num = nil)
       if num
         res = check_response(critical { get_response('UIDL %d', num) })
         return res.split(/ /)[1]
@@ -819,12 +959,12 @@ module Net
 
     private
 
-    def getok( fmt, *fargs )
+    def getok(fmt, *fargs)
       @socket.writeline sprintf(fmt, *fargs)
       check_response(recv_response())
     end
 
-    def get_response( fmt, *fargs )
+    def get_response(fmt, *fargs)
       @socket.writeline sprintf(fmt, *fargs)
       recv_response()
     end
@@ -833,13 +973,13 @@ module Net
       @socket.readline
     end
 
-    def check_response( res )
-      raise POPError, res unless /\A\+OK/i === res
+    def check_response(res)
+      raise POPError, res unless /\A\+OK/i =~ res
       res
     end
 
-    def check_response_auth( res )
-      raise POPAuthenticationError, res unless /\A\+OK/i === res
+    def check_response_auth(res)
+      raise POPAuthenticationError, res unless /\A\+OK/i =~ res
       res
     end
 
@@ -856,4 +996,3 @@ module Net
   end   # class POP3Command
 
 end   # module Net
-
