@@ -1,94 +1,98 @@
-# = net/telnet.rb - simple telnet client library
+# = net/telnet.rb - Simple Telnet Client Library
 # 
-# Wakou Aoyama <wakou@ruby-lang.org>
-#
-# == Overview
+# Author:: Wakou Aoyama <wakou@ruby-lang.org>
+# Documentation:: William Webber and Wakou Aoyama 
 #
 # This file holds the class Net::Telnet, which provides client-side
 # telnet functionality.
 #
-# The telnet protocol allows a client to login remotely to a user
-# account on a server and execute commands via a shell.  The equivalent
-# is done by creating a Net::Telnet class with the Host option
-# set to your host, calling #login() with your user and password,
-# issuing one or more #cmd() calls, and then calling #close()
-# to end the session.  The #waitfor(), #print(), #puts(), and
-# #write() methods, which #cmd() is implemented on top of, are
-# only needed if you are doing something more complicated.
+# For documentation, see Net::Telnet.
 #
-# A Net::Telnet object can also be used to connect to non-telnet
-# services, such as SMTP or HTTP.  In this case, you normally
-# want to provide the Port option to specify the port to
-# connect to, and set the Telnetmode option to false to prevent
-# the client from attempting to interpret telnet command sequences.
-# Generally, #login() will not work with other protocols, and you
-# have to handle authentication yourself.
-# For some protocols, it will be possible to specify the Prompt
-# option once when you create the Telnet object and use #cmd() calls; 
-# for others, you will have to specify the response sequence to
-# look for as the Match option to every #cmd() call, or call
-# #puts() and #waitfor() directly; for yet others, you will have 
-# to use #sysread() instead of #waitfor() and parse server 
-# responses yourself.
-#
-# It is worth noting that when you create a new Net::Telnet object,
-# you can supply a proxy IO channel via the Proxy option.  This
-# can be used to attach the Telnet object to other Telnet objects,
-# to already open sockets, or to any read-write IO object.  This
-# can be useful, for instance, for setting up a test fixture for
-# unit testing.
-# 
-# == Examples of use.
-# 
-# === Log in and send a command, echoing all output to stdout.
-# 
-#   localhost = Net::Telnet::new({"Host" => "localhost",
-#                                 "Timeout" => 10,
-#                                 "Prompt" => /[$%#>] \z/n})
-#   localhost.login("username", "password"){|c| print c }
-#   localhost.cmd("command"){|c| print c }
-#   localhost.close
-# 
-# 
-# === Check a POP server to see if you have mail.
-# 
-#   pop = Net::Telnet::new({"Host" => "your_destination_host_here",
-#                           "Port" => 110,
-#                           "Telnetmode" => false,
-#                           "Prompt" => /^\+OK/n})
-#   pop.cmd("user " + "your_username_here"){|c| print c}
-#   pop.cmd("pass " + "your_password_here"){|c| print c}
-#   pop.cmd("list"){|c| print c}
-#
-# == References.
-#
-# There are a large number of RFCs relevant to the Telnet protocol.
-# RFCs 854-861 define the base protocol.  For a complete listing
-# of relevant RFCs, see
-# http://www.omnifarious.org/~hopper/technical/telnet-rfc.html
-
 
 require "socket"
 require "delegate"
 require "timeout"
 require "English"
+ 
+module Net
 
-module Net # :nodoc:
-
+  #
+  # == Net::Telnet
+  #
   # Provides telnet client functionality.
   #
-  # This class also has, through delegation, all the methods of
-  # a socket object (by default, a TCPSocket, but can be set
-  # by the Proxy option to new()).  This provides methods
-  # such as #close() to end the session and #sysread() to
-  # read data directly from the host, instead of via the
-  # #waitfor() mechanism.  Note that if you do use #sysread() 
-  # directly when in telnet mode, you should probably pass
-  # the output through #preprocess() to extract telnet command
-  # sequences.
+  # This class also has, through delegation, all the methods of a
+  # socket object (by default, a +TCPSocket+, but can be set by the
+  # +Proxy+ option to <tt>new()</tt>).  This provides methods such as
+  # <tt>close()</tt> to end the session and <tt>sysread()</tt> to read
+  # data directly from the host, instead of via the <tt>waitfor()</tt>
+  # mechanism.  Note that if you do use <tt>sysread()</tt> directly
+  # when in telnet mode, you should probably pass the output through
+  # <tt>preprocess()</tt> to extract telnet command sequences.
+  #
+  # == Overview
+  #
+  # The telnet protocol allows a client to login remotely to a user
+  # account on a server and execute commands via a shell.  The equivalent
+  # is done by creating a Net::Telnet class with the +Host+ option
+  # set to your host, calling #login() with your user and password,
+  # issuing one or more #cmd() calls, and then calling #close()
+  # to end the session.  The #waitfor(), #print(), #puts(), and
+  # #write() methods, which #cmd() is implemented on top of, are
+  # only needed if you are doing something more complicated.
+  #
+  # A Net::Telnet object can also be used to connect to non-telnet
+  # services, such as SMTP or HTTP.  In this case, you normally
+  # want to provide the +Port+ option to specify the port to
+  # connect to, and set the +Telnetmode+ option to false to prevent
+  # the client from attempting to interpret telnet command sequences.
+  # Generally, #login() will not work with other protocols, and you
+  # have to handle authentication yourself.
+  #
+  # For some protocols, it will be possible to specify the +Prompt+
+  # option once when you create the Telnet object and use #cmd() calls; 
+  # for others, you will have to specify the response sequence to
+  # look for as the Match option to every #cmd() call, or call
+  # #puts() and #waitfor() directly; for yet others, you will have 
+  # to use #sysread() instead of #waitfor() and parse server 
+  # responses yourself.
+  #
+  # It is worth noting that when you create a new Net::Telnet object,
+  # you can supply a proxy IO channel via the Proxy option.  This
+  # can be used to attach the Telnet object to other Telnet objects,
+  # to already open sockets, or to any read-write IO object.  This
+  # can be useful, for instance, for setting up a test fixture for
+  # unit testing.
+  # 
+  # == Examples
+  # 
+  # === Log in and send a command, echoing all output to stdout
+  # 
+  #   localhost = Net::Telnet::new("Host" => "localhost",
+  #                                "Timeout" => 10,
+  #                                "Prompt" => /[$%#>] \z/n)
+  #   localhost.login("username", "password") { |c| print c }
+  #   localhost.cmd("command") { |c| print c }
+  #   localhost.close
+  # 
+  # 
+  # === Check a POP server to see if you have mail
+  # 
+  #   pop = Net::Telnet::new("Host" => "your_destination_host_here",
+  #                          "Port" => 110,
+  #                          "Telnetmode" => false,
+  #                          "Prompt" => /^\+OK/n)
+  #   pop.cmd("user " + "your_username_here") { |c| print c }
+  #   pop.cmd("pass " + "your_password_here") { |c| print c }
+  #   pop.cmd("list") { |c| print c }
+  #
+  # == References
+  #
+  # There are a large number of RFCs relevant to the Telnet protocol.
+  # RFCs 854-861 define the base protocol.  For a complete listing
+  # of relevant RFCs, see
+  # http://www.omnifarious.org/~hopper/technical/telnet-rfc.html
   #
-  # See the documentation to the telnet.rb file for an overview
-  # and examples of usage.
   class Telnet < SimpleDelegator
 
     # :stopdoc:
@@ -160,9 +164,10 @@ module Net # :nodoc:
     CR   = "\015"   
     LF   = "\012" 
     EOL  = CR + LF 
-    REVISION = '$Id: telnet.rb,v 1.23 2003/12/19 15:18:09 dave Exp $'
+    REVISION = '$Id: telnet.rb 16458 2008-05-18 15:02:36Z knu $'
     # :startdoc:
 
+    #
     # Creates a new Net::Telnet object.
     #
     # Attempts to connect to the host (unless the Proxy option is
@@ -176,7 +181,7 @@ module Net # :nodoc:
     # +options+ is a hash of options.  The following example lists
     # all options and their default values.
     # 
-    #   host = Net::Telnet::new({
+    #   host = Net::Telnet::new(
     #            "Host"       => "localhost",  # default: "localhost"
     #            "Port"       => 23,           # default: 23
     #            "Binmode"    => false,        # default: false
@@ -189,7 +194,7 @@ module Net # :nodoc:
     #            "Waittime"   => 0,            # default: 0
     #            "Proxy"      => proxy         # default: nil
     #                            # proxy is Net::Telnet or IO object
-    #          })
+    #          )
     #
     # The options have the following meanings:
     #
@@ -241,7 +246,7 @@ module Net # :nodoc:
     # Timeout:: the number of seconds to wait before timing out both the
     #           initial attempt to connect to host (in this constructor),
     #           and all attempts to read data from the host (in #waitfor(),
-    #           #cmd(), and #login()).  Exceding this timeout causes a
+    #           #cmd(), and #login()).  Exceeding this timeout causes a
     #           TimeoutError to be raised.  The default value is 10 seconds.
     #           You can disable the timeout by setting this value to false.
     #           In this case, the connect attempt will eventually timeout
@@ -266,6 +271,7 @@ module Net # :nodoc:
     #         instance will use that one's socket for communication.  If an
     #         IO object, it is used directly for communication.  Any other
     #         kind of object will cause an error to be raised.
+    #
     def initialize(options) # :yield: mesg 
       @options = options
       @options["Host"]       = "localhost"   unless @options.has_key?("Host")
@@ -514,10 +520,15 @@ module Net # :nodoc:
     #            value specified when this instance was created will be
     #            used, or, failing that, the default value of 0 seconds,
     #            which means not to wait for more input.
+    # FailEOF:: if true, when the remote end closes the connection then an
+    #           EOFError will be raised. Otherwise, defaults to the old
+    #           behaviour that the function will return whatever data
+    #           has been received already, or nil if nothing was received.
     #           
     def waitfor(options) # :yield: recvdata
       time_out = @options["Timeout"]
       waittime = @options["Waittime"]
+      fail_eof = @options["FailEOF"]
 
       if options.kind_of?(Hash)
         prompt   = if options.has_key?("Match")
@@ -529,6 +540,7 @@ module Net # :nodoc:
                    end
         time_out = options["Timeout"]  if options.has_key?("Timeout")
         waittime = options["Waittime"] if options.has_key?("Waittime")
+        fail_eof = options["FailEOF"]  if options.has_key?("FailEOF")
       else
         prompt = options
       end
@@ -545,7 +557,7 @@ module Net # :nodoc:
           raise TimeoutError, "timed out while waiting for more data"
         end
         begin
-          c = @sock.sysread(1024 * 1024)
+          c = @sock.readpartial(1024 * 1024)
           @dumplog.log_dump('<', c) if @options.has_key?("Dump_log")
           if @options["Telnetmode"]
             c = rest + c
@@ -553,7 +565,8 @@ module Net # :nodoc:
                Integer(c.rindex(/#{IAC}#{SB}/no))
               buf = preprocess(c[0 ... c.rindex(/#{IAC}#{SB}/no)])
               rest = c[c.rindex(/#{IAC}#{SB}/no) .. -1]
-            elsif pt = c.rindex(/#{IAC}[^#{IAC}#{AO}#{AYT}#{DM}#{IP}#{NOP}]?\z/no)
+            elsif pt = c.rindex(/#{IAC}[^#{IAC}#{AO}#{AYT}#{DM}#{IP}#{NOP}]?\z/no) ||
+                       c.rindex(/\r\z/no)
               buf = preprocess(c[0 ... pt])
               rest = c[pt .. -1]
             else
@@ -565,14 +578,21 @@ module Net # :nodoc:
            #
            # We cannot use preprocess() on this data, because that
            # method makes some Telnetmode-specific assumptions.
-           buf = c
-           buf.gsub!(/#{EOL}/no, "\n") unless @options["Binmode"]
+           buf = rest + c
            rest = ''
+           unless @options["Binmode"]
+             if pt = buf.rindex(/\r\z/no)
+               buf = buf[0 ... pt]
+               rest = buf[pt .. -1]
+             end
+             buf.gsub!(/#{EOL}/no, "\n")
+           end
           end
           @log.print(buf) if @options.has_key?("Output_log")
           line += buf
           yield buf if block_given?
         rescue EOFError # End of file reached
+          raise if fail_eof
           if line == ''
             line = nil
             yield nil if block_given?
@@ -698,27 +718,31 @@ module Net # :nodoc:
     # the host should not echo).  If a block is passed in, this received
     # data is also yielded to the block as it is received.
     def login(options, password = nil) # :yield: recvdata
+      login_prompt = /[Ll]ogin[: ]*\z/n
+      password_prompt = /[Pp]ass(?:word|phrase)[: ]*\z/n
       if options.kind_of?(Hash)
         username = options["Name"]
         password = options["Password"]
+	login_prompt = options["LoginPrompt"] if options["LoginPrompt"]
+	password_prompt = options["PasswordPrompt"] if options["PasswordPrompt"]
       else
         username = options
       end
 
       if block_given?
-        line = waitfor(/login[: ]*\z/n){|c| yield c }
+        line = waitfor(login_prompt){|c| yield c }
         if password
           line += cmd({"String" => username,
-                       "Match" => /Password[: ]*\z/n}){|c| yield c }
+                       "Match" => password_prompt}){|c| yield c }
           line += cmd(password){|c| yield c }
         else
           line += cmd(username){|c| yield c }
         end
       else
-        line = waitfor(/login[: ]*\z/n)
+        line = waitfor(login_prompt)
         if password
           line += cmd({"String" => username,
-                       "Match" => /Password[: ]*\z/n})
+                       "Match" => password_prompt})
           line += cmd(password)
         else
           line += cmd(username)
